--------------------------------------------------
-- ArekosGratz, Author: Areko                   --
--------------------------------------------------

ArekosGratz = LibStub("AceAddon-3.0"):NewAddon("ArekosGratz", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale("ArekosGratz");

local defaults = {
    profile = {
        protectAFK = true,
        congratulations = {
            guild = false,
            party = false,
            near = 0,
            guildInOtherChannel = false,
            minWaitTime = 2,
            maxWaitTime = 4,
        },
        thanks = {
            guild = false,
            party = false,
            near = 0,
            minWaitTime = 15,
            maxWaitTime = 20,        
        },
        gratzKeywords = nil,
        thxKeywords = nil
    }   
};

local gratzList = {
    profile = {
        "gz",
        "gw",
        "gg",
        "gzi",
        "grats",
        "gratz",
        "salute",    
        "congrat",
        "goncrats",
        "dingrats",
        "gradulate",
        "condolence",
        "coragulate",
        "congrations",
        "gratulation",
        "congrazzles",
        "congratsory",
        "glückwunsch",
        "felicitation",
        "concragulate",
        "congradulate",
        "comendations",
        "congraduation",   
        "confagulations",
        "congratulation",
        "congratulating",
        "congratulations",
        "concrapulations",
    }
}

local mainOption = {
    name = "ArekosGratz",
    type = "group",
    args = {
        protectAFK = {
            type    = "toggle",
            name    = L["Protect AFK-Status"],
            desc    = L["No congratulation while AFK"],
            order   = 1,
            set     = function(info, val) ArekosGratz.db.profile.protectAFK = val end,
            get     = function() return ArekosGratz.db.profile.protectAFK end,            
        },
        gratzDesc = {
            type    = "header",
            name    = L["Congratulate"],
            desc    = L["Settings for congratulations"],
            order   = 2
        },
        guildGratz = {
            type    = "toggle",
            name    = L["Guild"],
            desc    = L["Congratulations to the guild channel"],
            order   = 3,
            set     = function(info, val) ArekosGratz.db.profile.congratulations.guild = val end,
            get     = function() return ArekosGratz.db.profile.congratulations.guild end
        },
        partyGratz = {
            type    = "toggle",
            name    = L["Party"],
            desc    = L["Congratulations to the party channel"],
            order   = 3,
            set     = function(info, val) ArekosGratz.db.profile.congratulations.party = val end,
            get     = function() return ArekosGratz.db.profile.congratulations.party end
        },        
        nearGratz = {
            type    = "select",
            name    = L["Congratulations near players"],
            values  = {[0]=L["None"], [1]=L["Say"], [2]=L["Whisper"]};
            set     = function(info, val) ArekosGratz.db.profile.congratulations.near = val end,
            get     = function() return ArekosGratz.db.profile.congratulations.near end,
            style   = "radio",
            order   = 4
        },
        minGratz = {
            type    = "range",
            name    = L["Min. wait time to congratulation"],
            min     = 0,
            max     = 60,
            step    = 1,
            bigStep = 1,
            get     = function() return ArekosGratz.db.profile.congratulations.minWaitTime end,
            set     = function(info, val)  
                if ArekosGratz.db.profile.congratulations.maxWaitTime > val then
                    ArekosGratz.db.profile.congratulations.minWaitTime = val;
                else
                    return false;
                end
            end,
            order   = 5,
            width   = "double",
        },
        maxGratz = {
            type    = "range",
            name    = L["Max. wait time to congratulation"],
            min     = 0,
            max     = 60,
            step    = 1,
            bigStep = 1,
            get     = function() return ArekosGratz.db.profile.congratulations.maxWaitTime end,
            set     = function(info, val)  
                if ArekosGratz.db.profile.congratulations.minWaitTime < val then
                    ArekosGratz.db.profile.congratulations.maxWaitTime = val;
                else
                    return false;
                end
            end,
            order   = 5,
            width   = "double",
        },
        guildInOtherChannel = {
            type    = "toggle",
            name    = L["Congratulate guildmembers over other channels"],
            desc    = L["Congratulations to guildmembers who in our party or near your in other channels as guildchannel"],
            order   = 6,
            set     = function(info, val) ArekosGratz.db.profile.congratulations.guildInOtherChannel = val end,
            get     = function() return ArekosGratz.db.profile.congratulations.guildInOtherChannel end,
            width   = "full"
        },

        ThxDesc = {
            type    = "header",
            name    = L["Thanks"],
            desc    = L["Settings for saying thanks"],
            order   = 7
        },
        guildThx = {
            type    = "toggle",
            name    = L["Guild"],
            desc    = L["Saying thanks to the guild channel"],
            order   = 8,
            set     = function(info, val) ArekosGratz.db.profile.thanks.guild = val end,
            get     = function() return ArekosGratz.db.profile.thanks.guild end
        },
        partyThx = {
            type    = "toggle",
            name    = L["Party"],
            desc    = L["Saying thanks to the party channel"],
            order   = 8,
            set     = function(info, val) ArekosGratz.db.profile.thanks.party = val end,
            get     = function() return ArekosGratz.db.profile.thanks.party end
        },
        nearThx = {
            type    = "select",
            name    = L["Saying thanks near players"],
            values  = {[0]=L["None"], [1]=L["Say"], [2]=L["Whisper"]};
            set     = function(info, val) ArekosGratz.db.profile.thanks.near = val end,
            get     = function() return ArekosGratz.db.profile.thanks.near end,
            style   = "radio",
            order   = 9
        },
        minThx = {
            type    = "range",
            name    = L["Min. wait time to thanks"],
            min     = 0,
            max     = 60,
            step    = 1,
            bigStep = 1,
            get     = function() return ArekosGratz.db.profile.thanks.minWaitTime end,
            set     = function(info, val)  
                if ArekosGratz.db.profile.thanks.maxWaitTime > val then
                    ArekosGratz.db.profile.thanks.minWaitTime = val;
                else
                    return false;
                end
            end,
            order   = 10,
            width   = "double",
        },
        maxThx = {
            type    = "range",
            name    = L["Max. wait time to thanks"],
            min     = 0,
            max     = 60,
            step    = 1,
            bigStep = 1,
            get     = function() return ArekosGratz.db.profile.thanks.maxWaitTime end,
            set     = function(info, val)  
                if ArekosGratz.db.profile.thanks.minWaitTime < val then
                    ArekosGratz.db.profile.thanks.maxWaitTime = val;
                else
                    return false;
                end
            end,
            order   = 10,
            width   = "double",
        },
        
    },
};

--------------------------------------------------
-- Gratz option                                 --
--------------------------------------------------
local newGratzkeywordTemp = "";
local gratzOption = {
    name = "Congratulate option",
    type = "group",
    args = {
        AddNewGratzKeyword = {
            name    = L["Congratulations keywords"],
            desc    = L["Use #n for playername"],
            type    = "input",
            set     = function(info, val) newGratzkeywordTemp = val  end,
            get     = function() return newGratzkeywordTemp end,
        },
        AddNewGratzKeywordButton = {
            name    = L["Add Keyword"],
            type    = "execute",
            func    = function()
                if newGratzkeywordTemp == "" or ArekosGratz.db.profile.gratzKeywords[newGratzkeywordTemp] ~= nil then
                    return false
                end 
                ArekosGratz.db.profile.gratzKeywords[newGratzkeywordTemp] = {onlyFor = {}, notFor = {} };
                ArekosGratz:AddGratzKeyword(newGratzkeywordTemp);
            end,
        }
    }
};


function ArekosGratz:AddGratzKeyword(gratzKey)
    local editGratzkeywordTemp = nil;
    if gratzOption.args[gratzKey] == nil then
        gratzOption.args[gratzKey] = {
            name = gratzKey,
            type = "group",
            args = {
                deleteButton = {
                    type    = "execute",
                    name    = L["Delete"],
                    func    = function()
                        ArekosGratz.db.profile.gratzKeywords[gratzKey] = nil;
                        gratzOption.args[gratzKey] = nil;
                    end,
                    order   = 5
                },
                EditNewGratzKeyword = {
                    name    = L["Edit keyword"],
                    desc    = L["Use #n for playername"],
                    type    = "input",
                    set     = function(info, val) 
                        editGratzkeywordTemp = val;
                    end,
                    get     = function() return editGratzkeywordTemp or gratzKey end,
                    order   = 1,
                },
                EditNewGratzKeywordButton = {
                    name    = L["Edit keyword"],
                    type    = "execute",
                    func    = function()
                        if editGratzkeywordTemp == ""  or not editGratzkeywordTemp then
                            return false
                        end
                        ArekosGratz.db.profile.gratzKeywords[editGratzkeywordTemp] = ArekosGratz.db.profile.gratzKeywords[gratzKey];
                        ArekosGratz.db.profile.gratzKeywords[gratzKey] = nil;
                        gratzOption.args[gratzKey] = nil;

                        ArekosGratz:AddGratzKeyword(editGratzkeywordTemp);                        
                    end,
                    order   = 1,
                },
                onlyForPlayer = {
                    name    = L["Only for:"],
                    desc    = L["One player per line"],
                    type    = "input",
                    set     = function(info, val)
                        ArekosGratz.db.profile.gratzKeywords[gratzKey].onlyFor = {strsplit("\n", strlower(strtrim(val) ) ) };
                    end,
                    get     = function()
                        local list = {};
                        for k,v in pairs(ArekosGratz.db.profile.gratzKeywords[gratzKey].onlyFor) do
                            tinsert(list, v);
                        end
                        return strjoin("\n", unpack(list))
                    end,
                    multiline = true,
                    width   = "double",
                    order   = 2
                },
                notForPlayer = {
                    name    = L["Not for:"],
                    desc    = L["One player per line"],
                    type    = "input",
                    set     = function(info, val)
                        ArekosGratz.db.profile.gratzKeywords[gratzKey].notFor = {strsplit("\n", strlower(strtrim(val) ) )}
                    end,
                    get     = function()
                        local list = {};
                        for k,v in pairs(ArekosGratz.db.profile.gratzKeywords[gratzKey].notFor) do
                            tinsert(list, v);
                        end
                        return strjoin("\n", unpack(list))
                    end,
                    multiline = true,
                    width   = "double",
                    order   = 3
                },
            }
        }
    end

end

--------------------------------------------------
-- Thx option                                   --
--------------------------------------------------
local newThxKeywordTemp = "";
local thxOption = {
    name = "Congratulate option",
    type = "group",
    args = {
        AddNewThxKeyword = {
            name    = L["Congratulations keywords"],
            desc    = L["Use #n for playername"],
            type    = "input",
            set     = function(info, val) newThxKeywordTemp = val  end,
            get     = function() return newThxKeywordTemp end,
        },
        AddNewThxKeywordButton = {
            name    = L["Add Keyword"],
            type    = "execute",
            func    = function()
                if newThxKeywordTemp == "" or ArekosGratz.db.profile.thxKeywords[newThxKeywordTemp] ~= nil then
                    return false
                end 
                ArekosGratz.db.profile.thxKeywords[newThxKeywordTemp] = {onlyFor = {}, notFor = {} };
                ArekosGratz:AddThxKeyword(newThxKeywordTemp);
            end,
        }
    }
};

function ArekosGratz:AddThxKeyword(thxKey)
    local editThxkeywordTemp = nil;
    if thxOption.args[thxKey] == nil then
        thxOption.args[thxKey] = {
            name = thxKey,
            type = "group",
            args = {
                deleteButton = {
                    type    = "execute",
                    name    = L["Delete"],
                    func    = function()
                        ArekosGratz.db.profile.thxKeywords[thxKey] = nil;
                        thxOption.args[thxKey] = nil;
                    end,
                    order   = 5
                },
                EditNewThxKeyword = {
                    name    = L["Edit keyword"],
                    desc    = L["Use #n for playername"],
                    type    = "input",
                    set     = function(info, val) 
                        editThxkeywordTemp = val;
                    end,
                    get     = function() return editThxkeywordTemp or thxKey end,
                    order   = 1,
                },
                EditNewThxKeywordButton = {
                    name    = L["Edit keyword"],
                    type    = "execute",
                    func    = function()
                        if editThxkeywordTemp == "" or not editThxkeywordTemp then
                            return false
                        end
                        ArekosGratz.db.profile.thxKeywords[editThxkeywordTemp] = ArekosGratz.db.profile.thxKeywords[thxKey];                        
                        ArekosGratz.db.profile.thxKeywords[thxKey] = nil;
                        thxOption.args[thxKey] = nil;
                        ArekosGratz:AddThxKeyword(editThxkeywordTemp);
                    end,
                    order   = 1,
                },
                onlyForPlayer = {
                    name    = L["Only for:"],
                    desc    = L["One player per line"],
                    type    = "input",
                    set     = function(info, val)
                        ArekosGratz.db.profile.thxKeywords[thxKey].onlyFor = {strsplit("\n", strlower(strtrim(val) ) ) };
                    end,
                    get     = function()
                        local list = {};
                        for k,v in pairs(ArekosGratz.db.profile.thxKeywords[thxKey].onlyFor) do
                            tinsert(list, v);
                        end
                        return strjoin("\n", unpack(list))
                    end,
                    multiline = true,
                    width   = "double",
                    order   = 2
                },
                notForPlayer = {
                    name    = L["Not for:"],
                    desc    = L["One player per line"],
                    type    = "input",
                    set     = function(info, val)
                        ArekosGratz.db.profile.thxKeywords[thxKey].notFor = {strsplit("\n", strlower(strtrim(val)) )}
                    end,
                    get     = function()
                        local list = {};
                        for k,v in pairs(ArekosGratz.db.profile.thxKeywords[thxKey].notFor) do
                            tinsert(list, v);
                        end
                        return strjoin("\n", unpack(list))
                    end,
                    multiline = true,
                    width   = "double",
                    order   = 3
                },
            }
        }
    end

end


local gratzListOption = {
    name = "GratzList",
    type = "group",
    args = {
        list = {
            name    = L["Congratulations Keywords"],
            desc    = L["List of triggers for thanks\nOne word per line"],
            type    = "input",
            set     = function(info, val)
                ArekosGratz.gratzDB.profile = {strsplit("\n", strlower(strtrim(val) ) )};
            end,
            get     = function()
                local list = {};
                for k,v in pairs(ArekosGratz.gratzDB.profile) do
                    tinsert(list, v);
                end
                return strjoin("\n", unpack(list))
            end,
            multiline = true,
            width   = "full",

            order   = 1
        },
    }
};


function ArekosGratz:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("ArekosGratzDB", defaults, true);
    self.gratzDB = LibStub("AceDB-3.0"):New("GratzDB", gratzList, true);
    self.profileOptions = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db);    

    if self.db.profile.gratzKeywords == nil then
        self.db.profile.gratzKeywords = {
            gz = {
                onlyFor = {},
                notFor = {}
            }
        };
    end
    if self.db.profile.thxKeywords == nil then
        self.db.profile.thxKeywords = {
            thx = {
                onlyFor = {},
                notFor = {}
            }
        };
    end

    local config = LibStub("AceConfig-3.0");
    local registry = LibStub("AceConfigRegistry-3.0");
    local dialog = LibStub("AceConfigDialog-3.0");

    for k,v in pairs(self.db.profile.gratzKeywords) do
        self:AddGratzKeyword(k);
    end

    for k,v in pairs(self.db.profile.thxKeywords) do
        self:AddThxKeyword(k);
    end

    registry:RegisterOptionsTable("ArekosGratz", mainOption);
    registry:RegisterOptionsTable("ArekosGratz Gratz", gratzOption);
    registry:RegisterOptionsTable("ArekosGratz Thx", thxOption);
    registry:RegisterOptionsTable("ArekosGratz GratzDB", gratzListOption);
    registry:RegisterOptionsTable("ArekosGratz Profile", self.profileOptions);

    self.optionsFrame = {
        main = dialog:AddToBlizOptions("ArekosGratz", L["ArekosGratz"]),
        gratz = dialog:AddToBlizOptions("ArekosGratz Gratz", L["Congratulate"], L["ArekosGratz"]),
        thx = dialog:AddToBlizOptions("ArekosGratz Thx", L["Thanks"], L["ArekosGratz"]),
        gratzDB = dialog:AddToBlizOptions("ArekosGratz GratzDB", L["Thanks Triggers"], L["ArekosGratz"]),
        profile = dialog:AddToBlizOptions("ArekosGratz Profile", L["Profile"], L["ArekosGratz"])
    };


    self:RegisterChatCommand("arekosgratz", "ChatCommand")
    self:RegisterChatCommand("ag", "ChatCommand")

    self:RegisterEvent("PLAYER_ENTERING_WORLD");
    self:RegisterEvent("PLAYER_GUILD_UPDATE");
    self:RegisterEvent("GUILD_ROSTER_UPDATE");
    self:RegisterEvent("ACHIEVEMENT_EARNED");
    self:RegisterEvent("CHAT_MSG_ACHIEVEMENT");
    self:RegisterEvent("CHAT_MSG_GUILD_ACHIEVEMENT");
    self:RegisterEvent("CHAT_MSG_GUILD");
    self:RegisterEvent("CHAT_MSG_PARTY");
    self:RegisterEvent("CHAT_MSG_PARTY_LEADER");
    self:RegisterEvent("CHAT_MSG_RAID");
    self:RegisterEvent("CHAT_MSG_RAID_LEADER");
    self:RegisterEvent("CHAT_MSG_SAY");
    self:RegisterEvent("CHAT_MSG_WHISPER");
end


function ArekosGratz:OnEnable()
    self.playername = UnitName("player");    
end

function ArekosGratz:OnDisable()
    
end

function ArekosGratz:PLAYER_ENTERING_WORLD()
    self.guildname = GetGuildInfo("player") or "";
end
function ArekosGratz:GUILD_ROSTER_UPDATE()
    self.guildname = GetGuildInfo("player") or "";
end
function ArekosGratz:PLAYER_GUILD_UPDATE()
    self.guildname = GetGuildInfo("player") or "";
end

function ArekosGratz:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame);
    else
        LibStub("AceConfigCmd-3.0").HandleCommand("ArekosGratz", "ag", "ArekosGratz", input);
    end
end

--[[
ACHIEVEMENT_EARNED, a1=achievementid
CHAT_MSG_ACHIEVEMENT, a1=message,a2=playername
CHAT_MSG_GUILD_ACHIEVEMENT, a1=message,a2=playername,a12=GUID


CHAT_MSG_GUILD, a1=message,a2=user
CHAT_MSG_PARTY, a1=message,a2=user
CHAT_MSG_PARTY_LEADER, a1=message,a2=user
CHAT_MSG_RAID, a1=message,a2=user
CHAT_MSG_RAID_LEADER, a1=message,a2=user
CHAT_MSG_SAY, a1=message,a2=user
CHAT_MSG_WHISPER, a1=message,a2=user
]]

local guildPendingList = {thx = {}, gz = {}};
local guildTimer = nil;
local partyPendingList = {thx = {}, gz = {}};
local partyTimer = nil;
local nearPendingList = {thx = {}, gz = {}};
local nearTimer = nil;
local thxTimer = nil;

function ArekosGratz:ACHIEVEMENT_EARNED( ... )
    
    if not self:TimeLeft(thxTimer) or 0 > 0 then
        thxTimer = self:ScheduleTimer("SayThanks", random(self.db.profile.thanks.minWaitTime, self.db.profile.thanks.maxWaitTime) );
    end
end

function ArekosGratz:CHAT_MSG_ACHIEVEMENT( ... )
    if strlower(select(3, ...) ) == strlower(self.playername) or strlower(select(3, ...)) == strlower(self.guildname) then return end
    
    if not self.db.profile.congratulations.guildInOtherChannel and UnitIsInMyGuild(strlower(select(3, ...)) ) then
        return
    end

    if UnitInParty(tostring(select(3, ...)) ) or UnitInRaid(tostring(select(3, ...)) ) then
        if not self:TimeLeft(partyTimer) or 0 > 0 then
            partyTimer = self:ScheduleTimer("SayGratz", random(self.db.profile.congratulations.minWaitTime, self.db.profile.congratulations.maxWaitTime), "party");
        end
        
        tinsert(partyPendingList.gz, tostring(select(3, ...) ) );
    else
        if not self:TimeLeft(nearTimer) or 0 > 0 then
            nearTimer = self:ScheduleTimer("SayGratz", random(self.db.profile.congratulations.minWaitTime, self.db.profile.congratulations.maxWaitTime), "near");
        end
        
        tinsert(nearPendingList.gz, tostring(select(3, ...) ) );
    end
end

function ArekosGratz:CHAT_MSG_GUILD_ACHIEVEMENT( ... )
    if strlower(select(3, ...) ) == strlower(self.playername) or strlower(select(3, ...)) == strlower(self.guildname) then return end

    if not self:TimeLeft(guildTimer) or 0 > 0 then
        guildTimer = self:ScheduleTimer("SayGratz", random(self.db.profile.congratulations.minWaitTime, self.db.profile.congratulations.maxWaitTime), "guild");
    end
    
    tinsert(guildPendingList.gz, tostring(select(3, ...) ) );
end


function ArekosGratz:CHAT_MSG_GUILD( ... )    
    if strlower(select(3, ...) ) ~= strlower(self.playername) and self:CheckMessage(select(2, ...)) and self:TimeLeft(thxTimer) or 0 > 0 then 
        tinsert(guildPendingList.thx, tostring(select(3, ...) ));
    end
end

function ArekosGratz:CHAT_MSG_PARTY( ... )
    if strlower(select(3, ...) ) ~= strlower(self.playername) and  self:CheckMessage(select(2, ...)) and self:TimeLeft(thxTimer) or 0 > 0 then 
        tinsert(partyPendingList.thx, tostring(select(3, ...) ));
    end
end

function ArekosGratz:CHAT_MSG_PARTY_LEADER( ... )
    if strlower(select(3, ...) ) ~= strlower(self.playername) and  self:CheckMessage(select(2, ...)) and self:TimeLeft(thxTimer) or 0 > 0 then 
        tinsert(partyPendingList.thx, tostring(select(3, ...) ));
    end
end

function ArekosGratz:CHAT_MSG_RAID( ... )
    if strlower(select(3, ...) ) ~= strlower(self.playername) and  self:CheckMessage(select(2, ...)) and self:TimeLeft(thxTimer) or 0 > 0 then 
        tinsert(partyPendingList.thx, tostring(select(3, ...) ));
    end
end

function ArekosGratz:CHAT_MSG_RAID_LEADER( ... )
    if strlower(select(3, ...) ) ~= strlower(self.playername) and  self:CheckMessage(select(2, ...)) and self:TimeLeft(thxTimer) or 0 > 0 then 
        tinsert(partyPendingList.thx, tostring(select(3, ...) ));
    end
end

function ArekosGratz:CHAT_MSG_SAY( ... )
    if strlower(select(3, ...) ) ~= strlower(self.playername) and  self:CheckMessage(select(2, ...)) and self:TimeLeft(thxTimer) or 0 > 0 then 
        tinsert(nearPendingList.thx, tostring(select(3, ...) ));
    end
end

function ArekosGratz:CHAT_MSG_WHISPER( ... )
    if strlower(select(3, ...) ) ~= strlower(self.playername) and  self:CheckMessage(select(2, ...)) and self:TimeLeft(thxTimer) or 0 > 0 then 
        tinsert(nearPendingList.thx, tostring(select(3, ...) ));
    end
end

function ArekosGratz:SayThanks()
    thxTimer = nil;
    if not (self.db.profile.protectAFK and UnitIsAFK("player") ) then
        if self.db.profile.thanks.guild then
            local message = self:GetMessagesTable(self.db.profile.thxKeywords, ArekosGratz:removeDuplicate(guildPendingList.thx) );
            for i=1,#message do
                self:ScheduleTimer("SendChatMessage", i, {message[i], "GUILD"});
            end
        end
        if self.db.profile.thanks.party then
            local message = self:GetMessagesTable(self.db.profile.thxKeywords, ArekosGratz:removeDuplicate(partyPendingList.thx) );
            
            local channel = (IsInRaid() and "RAID" or "PARTY");
            for i=1,#message do
                self:ScheduleTimer("SendChatMessage", i, {message[i], channel});
            end

        end
        if self.db.profile.thanks.near ~= 0 then
            if self.db.profile.thanks.near == 1 then
                local message = self:GetMessagesTable(self.db.profile.thxKeywords, ArekosGratz:removeDuplicate(nearPendingList.thx) );
                
                for i=1,#message do
                    self:ScheduleTimer("SendChatMessage", i, {message[i], "SAY"});
                end

            elseif self.db.profile.thanks.near == 2 then
                for k,v in pairs(ArekosGratz:removeDuplicate(nearPendingList.thx) ) do
                    local message = self:GetMessagesTable(self.db.profile.thxKeywords, {v} );
                    
                    for i=1,#message do
                        self:ScheduleTimer("SendChatMessage", i, {message[i], "WHISPER", v});
                    end

                end
            end            
        end
    else
        thxTimer = nil;
    end
    guildPendingList.thx = {};
    partyPendingList.thx = {};
    nearPendingList.thx = {};
end

function ArekosGratz:SayGratz(timer)
    
    
    if not (self.db.profile.protectAFK and UnitIsAFK("player") ) then
        if timer == "guild" and self.db.profile.congratulations.guild then
            guildTimer = nil;
            local message = self:GetMessagesTable(self.db.profile.gratzKeywords, ArekosGratz:removeDuplicate(guildPendingList.gz) );

            for i=1,#message do
                self:ScheduleTimer("SendChatMessage", i, {message[i], "GUILD"});
            end        

            guildPendingList.gz = {};    
        end
        if timer == "party" and self.db.profile.congratulations.party then
            partyTimer = nil;
            local message = self:GetMessagesTable(self.db.profile.gratzKeywords, ArekosGratz:removeDuplicate(partyPendingList.gz) );
            
            local channel = (IsInRaid() and "RAID" or "PARTY");
            for i=1,#message do
                self:ScheduleTimer("SendChatMessage", i, {message[i], channel});
            end

            partyPendingList.gz = {};    
        end
        if timer == "near" and self.db.profile.congratulations.near ~= 0 then
            nearTimer = nil;
            if self.db.profile.congratulations.near == 1 then
                local message = self:GetMessagesTable(self.db.profile.gratzKeywords, ArekosGratz:removeDuplicate(nearPendingList.gz) );
                
                for i=1,#message do
                    self:ScheduleTimer("SendChatMessage", i, {message[i], "SAY"});
                end

            elseif self.db.profile.congratulations.near == 2 then
                for k,v in pairs(ArekosGratz:removeDuplicate(nearPendingList.gz) ) do
                    local message = self:GetMessagesTable(self.db.profile.gratzKeywords, {v} );

                    for i=1,#message do
                        self:ScheduleTimer("SendChatMessage", i, {message[i], "WHISPER", v});
                    end
                end
            end
            nearPendingList.gz = {};
        end
    else
        if timer == "guild" then
            guildTimer = nil;
        end
        if timer == "party" then
            partyTimer = nil;
        end
        if timer == "near" then
            nearTimer = nil;
        end
    end    
end

function  ArekosGratz:SendChatMessage(msgTable)
    local msg, chatType, to = unpack(msgTable);
    if msg == "" then return end

    if to then
        SendChatMessage(msg ,chatType , nil, to);
        
    else
        SendChatMessage(msg ,chatType);
        
    end
    
end

function ArekosGratz:GetMessagesTable(rules, nameList)
    local message = {};
    local specialGroupName = {};

    if getn(nameList) > 1 then
        local globalRule = {};
        for ruleName,rule in pairs(rules) do
            if getn(rule.onlyFor) == 0 then
                tinsert(globalRule, ruleName)            
            end
        end
        
        local id = random(1,getn(globalRule) );
        local allrule = rules[globalRule[id] ];
        local ruleName = globalRule[id];
        local allGroupName = {};
        
        for _,name in pairs(nameList) do
            if tContains(allrule.notFor, strlower(strtrim(name) ) ) then
                tinsert(specialGroupName, name);
            else
                tinsert(allGroupName, name);
            end
        end
        
        if getn(allGroupName) > 0 then
            local m = gsub(ruleName, "#n", strjoin(", ", unpack(allGroupName) ) );
            tinsert(message,  m);
        end   

    else
        specialGroupName = nameList;
    end

    local list = {};
    repeat        
        for ruleName,rule in pairs(rules) do
            list[ruleName] = {};
            

            for _,name in pairs(specialGroupName) do
                if tContains(rule.onlyFor, strlower(strtrim(name)) ) then
                    tinsert(list[ruleName], name);
                end
            end


        end

        local greatestList = {};
        local ruleName = "";
        for k,names in pairs(list) do            
            if getn(greatestList) < getn(names) then
                greatestList = names;
                ruleName = k;
            end
        end

        local newGroupName = {};            
        for k,name in pairs(specialGroupName) do 
            if tContains(greatestList, name) then
                tinsert(newGroupName, name);
            end
        end
        specialGroupName = newGroupName;

        if getn(greatestList) > 0 then
            local m = gsub(ruleName, "#n", strjoin(", ", unpack(greatestList) ) );
            tinsert(message,  m);
        end

    until getn(list) <= 1

    if getn(message) == 0 and getn(nameList) == 1 then
        local globalRule = {};
        for ruleName,rule in pairs(rules) do
            if getn(rule.onlyFor) == 0 then
                tinsert(globalRule, ruleName)            
            end
        end
        
        local id = random(1,getn(globalRule) );
        local allrule = rules[globalRule[id] ];
        local ruleName = globalRule[id];
        local allGroupName = {};

        for _,name in pairs(nameList) do
            if tContains(allrule.notFor, strlower(strtrim(name) ) ) then
                
            else
                tinsert(allGroupName, name);
            end
        end
        
        if getn(allGroupName) > 0 then
            local m = gsub(ruleName, "#n", strjoin(", ", unpack(allGroupName) ) );
            tinsert(message,  m);
        end
    end

    return message;
end

function ArekosGratz:removeDuplicate(table)
    local t1 = {};
    local clean = {};
    for k,v in pairs(table) do
        if t1[v] == nil then
            tinsert(clean, v);
        end
        t1[v] = true;
    end

    return clean;
end

function ArekosGratz:CheckMessage(msg)
    return tContains(self.gratzDB.profile, strlower(strtrim(msg)) );
end
